#[macro_use]
extern crate rocket;
#[macro_use]
extern crate diesel_migrations;

use figment::{
    providers::{Env, Format, Yaml},
    Figment,
};
use rb::auth::JwtConf;
use rocket::{
    fairing::AdHoc,
    http::Status,
    serde::json::{json, Value},
    Build, Request, Rocket,
};
use rocket_sync_db_pools::database;
use serde::{Deserialize, Serialize};

pub mod v1;

/// Used by Rocket to store database connections.
#[database("postgres_rb")]
pub struct RbDbConn(diesel::PgConnection);

/// Handles all error status codes.
#[catch(default)]
fn default_catcher(status: Status, _: &Request) -> Value
{
    json!({"status": status.code, "message": ""})
}

/// Rocket fairing that executes the necessary migrations in our database.
async fn run_db_migrations(rocket: Rocket<Build>) -> Result<Rocket<Build>, Rocket<Build>>
{
    embed_migrations!();

    let conn = RbDbConn::get_one(&rocket)
        .await
        .expect("database connection");
    conn.run(|c| match embedded_migrations::run(c) {
        Ok(()) => Ok(rocket),
        Err(_) => Err(rocket),
    })
    .await
}

/// Struct to deserialize from the config file. It contains any custom configuration our
/// application might need besides the default Rocket variables.
#[derive(Debug, Deserialize, Serialize)]
pub struct RbConfig
{
    jwt: JwtConf,
}

/// The main entrypoint of our program. It launches the Rocket instance.
#[launch]
fn rocket() -> _
{
    let figment = Figment::from(rocket::config::Config::default())
        .merge(Yaml::file("Rb.yaml").nested())
        .merge(Env::prefixed("RB_").global());

    let rocket = rocket::custom(figment)
        .attach(RbDbConn::fairing())
        .attach(AdHoc::try_on_ignite(
            "Run database migrations",
            run_db_migrations,
        ))
        .register("/", catchers![default_catcher]);

    let new_figment = rocket.figment();
    let jwt_conf: JwtConf = new_figment.extract_inner("jwt").expect("jwt config");

    rocket.manage(jwt_conf)
}
